/*
	Copyright 2014 Michele "King_DuckZ" Santullo

	This file is part of CloonelJump.

	CloonelJump is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CloonelJump is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CloonelJump.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef id17908979556C47F8A978688BBE4A9D22

#include "placeable.hpp"
#include "collidertypedef.hpp"
#include <memory>

namespace cloonel {
	class SDLMain;
	class GameplayScene;
	class Collider;

	class PlatformSystem : public Placeable {
	public:
		PlatformSystem ( void ) = delete;
		PlatformSystem ( const char* parTexturePath, SDLMain* parSDLMain, GameplayScene* parScene, float parMaxDistance );
		PlatformSystem ( const PlatformSystem& ) = delete;
		PlatformSystem ( PlatformSystem&& parOther ) = delete;
		~PlatformSystem ( void ) noexcept;
		PlatformSystem& operator= ( const PlatformSystem& ) = delete;

		void Prepare ( void );
		void RegisterForCollision ( ColliderRegisterFunc parReg, ColliderUnregisterFunc parUnreg );
		void AddDrawables ( void );
		void Destroy ( void ) noexcept;
		void SpawnPlatforms ( void );

		//Overrides
		virtual void OnRegister ( Mover& parOut, Mover::PlaceableTicketType parTicket );

	private:
		struct LocalData;

		const std::unique_ptr<LocalData> m_localdata;
		ColliderRegisterFunc m_registerToCollider;
		ColliderUnregisterFunc m_unregisterFromCollider;
	};
} //namespace cloonel

#endif
